<?php

if ( ! @include_once( './mainfile.php' ) ) { exit( '!! INCLUDE ERROR. ACCESS DENIED !!' ); }
error_reporting( _E_VALUE );


/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +----------------------------------------------------------------------+
// | BBCode Secured Sanity - BBcode Input / Output Sanitizing             |
// +----------------------------------------------------------------------+
// | Copyright (c) 2003 Eric Haddix                                       |
// +----------------------------------------------------------------------+
// | License: GNU/GPL - http://www.gnu.org/copyleft/gpl.html              |
// +----------------------------------------------------------------------+
// | Support: http://myphpnuke.com/                                       |
// +----------------------------------------------------------------------+
// | Required Files: core_Sanitize.class.php                              |
// |                 core_Sanitize.param.php                              |
// |                 core_AllowArrays.param.php ( MPN Option )            |
// +----------------------------------------------------------------------+
// | Class Author: Kiril Threndor <mpnx@dev.illearth.net>                 |
// +----------------------------------------------------------------------+
// | NOTES: Extends class Sanitize and is used to encode / decode forum   |
// | messages and other bbcode text                                       |
// +----------------------------------------------------------------------+
// | IMPORTANT:                                                           |
// |   This Subclass Is Intended For Secondary Sanitizing.                |
// |                                                                      |
// |   Specific checks were left out of this class in order to            |
// |   increase perfomance and to avoid redundant checks that the         |
// |   CheckInput Subclass already covered.                               |
// |   Unexpected results could occur if you don't run CheckInput first.  |
// +----------------------------------------------------------------------+
// $Id: core_BBCodeConvert.subclass.php,v 1.11 2003/11/15 04:36:27 kirilt Exp $

class BBCodeConvert extends Sanitizer
{
    /**
     * Enable / Disable debugging
     *
     * @var bool
     */

    var $debug                 = FALSE;

    /**
     * Enable / Disable tag closing
     *
     * Tags set in self_closing_tags array
     *
     * @var bool
     */

    var $force_tag_closing     = TRUE;

    /**
     * Add User Signature
     * Determined In Mainfile
     *
     * @var bool
     */

    var $addsig                = FALSE;

    /**
     * Add Attributes To Tags
     *
     * @var bool
    */

    var $add_attrib            = TRUE;

    /**
     * BBCode Swap
     *
     * The array of bb tags and their corresponding HTML tag
     * Array Entries: bbtag => html replacement tag
     *
     * @var array
     */

    var $bb_swap               = array();

    /**
     * Re-initializing the parent variable
     *
     * @var array
     */

    var $strip_tags            = array();

    /**
     * Protect Content Tag Array
     * Default: Protect None
     *
     * @var array
     */

    var $protect_content       = array( 'code' );

    /**
     * Allow HTML
     * Default: NO
     *
     * @var bool
     */

    var $allow_html            = FALSE;

    /**
     * Allow BBCode
     * Default: Yes
     *
     * @var bool
     */

    var $allow_bbcode          = TRUE;

    /**
     * Subclass BBCodeConvert Constructor
     *
     * @param   $allow_html     ( Bool  -> allow / disallow html, Default TRUE )
     *          $allow_bbcode   ( Bool  -> allow / disallow bbcode, Default TRUE )
     *          $addsig         ( Bool  -> Add Users Signature, Default FALSE )
     *          $param_arrays   ( Array -> Custom Runtime Parameter Arrays, Default NULL )
     *          $allowed_bbtags ( Array -> Custom Runtime BBCode Allowed Tag Array, Default NULL )
     *
     * @return  void
     */

    function BBCodeConvert( $allow_html = TRUE, $allow_bbcode = TRUE, $addsig = FALSE, $param_arrays = NULL, $allowed_bbtags = NULL )
    {
        // Check Incoming Runtime Arguments And Adjust If Necessary

        if ( ! empty( $param_arrays ) )
        {
            require $param_arrays;
        }
        else
        {
            require './classes/subs/sanitizer/core_BBCodeConvert.param.php';
        }

        // Allow / Disallow HTML Tags, Strip tags and content if FALSE

        if ( (bool)TRUE === $allow_html )
        {
            $this -> allow_html = (bool)TRUE;
        }

        // Allow / Disallow BBCode Tags, Strip tags and content if FALSE

        if ( (bool)FALSE == $allow_bbcode )
        {
            $this -> allow_bbcode = (bool)FALSE;
        }

        // Add User Signature

        if ( $addsig ) { $this -> addsig = (bool)TRUE; }

        // Runtime Allowed BBCode Tag Array ( Resets $bb_tag Array From Param File )

        if ( empty( $_SERVER['SAN'] -> bbtaglist ) )
        {
            $this -> taglist = ( ! empty( $allowed_bbtags ) ) ? ( $allowed_bbtags ) : ( $bb_tags );
        }
        else
        {
            $this -> taglist = $_SERVER['SAN'] -> bbtaglist;
        }

        if ( ! empty( $this -> taglist ) )
        {
            if ( is_bool( $this -> taglist[0] ) )
            {
                $this -> rmtags = array_shift( $this -> taglist );

            }

            // Normalize

            array_walk( $this -> taglist, array( $this, 'casenormalize' ) );
        }

        // Holds The BBCode Swap Tags

        if ( ! empty( $bb_swap ) ) { $this -> bb_swap = $bb_swap; }

        // Add Attributes To Tags

        if ( ! empty( $add_attr_to_tag ) ) { $this -> add_attr_to_tag = $add_attr_to_tag; }

        return;

    } // End Constructor

    /**
     * BBEncodes / BBDecodes Body Of Text
     *
     * It checks both HTML and BBCode Style Tags
     *
     * @param   $body           ( string -> content to process )
     *          $encode         ( bool   -> encode(TRUE) / decode(FALSE) content )
     *          $edited         ( bool   -> message is being edited )
     *          $editby         ( bool   -> message edited by username )
     *
     * @return  string -> Encoded / Decoded String
    */

    function BBConversion( $body, $encode, $edited = FALSE, $editby = '' )
    {
        // Set Encoding / Decoding Conditions & Variables

        if ( $encode )
        {
            $encode = (bool)TRUE;
            $print_tag = $this -> bb_swap;
            $delim = $this -> html_delim;

            // If Not Returning Literally, Set strip_tags Array

            if ( ! $this -> return_literal )
            {
                switch( (bool)TRUE )
                {
                    case ( $this -> allow_html && $this -> allow_bbcode ):

                        $this -> strip_tags = array();

                    break;

                    case ( ! $this -> allow_html ):

                        $this -> strip_tags = array( '<' );

                    break;

                    case ( ! $this -> allow_bbcode ):

                        $this -> strip_tags = array( '[' );

                    break;

                    case ( ! $this -> allow_html && ! $this -> allow_bbcode ):

                        $this -> strip_tags = array( '<', '[' );

                    break;

                    default:

                        $this -> strip_tags = array( '<' );

                    break;

                } // End Strip Switch
            }
            else
            {
                $this -> strip_tags = array();
            }
        }
        else // Decoding
        {
            $encode = (bool)FALSE;
            $this -> taglist = $this -> bb_swap;
            $print_tag = array_flip( $this -> bb_swap );
            $delim = $this -> bb_delim;
            $this -> strip_tags = array( '[' );
        }

        // Adjust Format For Parsing

        $this -> adjustFormat( $body, 3, 11 );

        // Initialize Variables

        $me = 'BBConversion';

        $curpos = 0;
        $trusted = '';
        $open_tags = array();
        $skip_content = (bool)FALSE;
        $this -> body_length =strlen( $body );

        //$this -> spew( "$me: invoking the loop\n" );

        // Process The Body

        while ( (bool)FALSE !== ( $curtag = $this -> getnxtag( $body, $curpos ) ) )
        {
            list( $tagname, $attary, $tagtype, $lt, $gt ) = $curtag;

            /**
             * Append Free Content To Trusted
             */

            if ( (bool)FALSE === $skip_content )
            {
                $cleanup = ( $curpos == 0 ) ? ( (bool)TRUE ) : ( (bool)FALSE );
                $skip_action = ( $tagtype == 2 && in_array( $tagname, $this -> skip_action_tag ) )
                             ? ( (bool)TRUE )
                             : ( (bool)FALSE );

                $trusted .= $this -> freeContent( $body, $lt, $curpos, $cleanup, $skip_action );
            }
            else
            {
                //$this -> spew( "$me: Skipping free content.\n" );
            }

            /**
             * If Tag Delimiter Is In The Strip By Delimiter Array,
             * Remove The Tag And Content Unconditionally
             * Additionally Remove Any Content And Tags That Are Not Allowed
             */

            if ( (bool)FALSE !== $skip_content || ! isset( $print_tag[ $tagname ] ) || in_array( substr( $body, $lt, 1 ), $this -> strip_tags ) )
            {
                $curpos = $gt + 1;

                if ( $skip_content === (bool)FALSE )
                {
                    if ( $tagtype != 3 )
                    {
                        // Found Opening Tag, Start Content Skipping

                        //$this -> spew( "$me: <b>Removing Invalid Tagset <i>{$skip_content}</i> With Content</b>\n" );

                        $skip_content = $tagname;
                    }
                    else // Found A Self-Closing Tag, Just Skip The Tag
                    {
                        //$this -> spew( "$me: <b>Removing Invalid Type 3 Tag <i>{$tagname}</i></b>\n" );
                    }
                }
                else
                {
                    if ( $skip_content == $tagname && $tagtype == 2 )
                    {
                        // Found the Closing Tag, Stop Skipping And Continue

                        //$this -> spew( "$me: <b>Finished Removing Invalid Tagset <i>{$skip_content}</i></b>.  Continuing With Normal Scanning\n" );

                        $skip_content = (bool)FALSE;
                    }
                }

                continue;
            }

            /**
             * If Decoding, Check If We Have Any Protected Content Present In Body
             * If We Do, Get the Protected Content and Append To Trusted
             * Additionally, We Bypass Any Further Action On That Content
             */

            if (    (bool)FALSE === $skip_content
                 && $tagtype == 1
                 && ! empty( $this -> protect_content )
                 && in_array( $tagname, $this -> protect_content )
               )
            {
                $protect = $this -> getProtectedContent( $body, $lt, $tagname, $delim );

                if ( (bool)FALSE !== $protect )
                {
                    //$this -> spew( "$me: Skipping Over Previous Protected Content And Appending " . strlen( $protect[0] ) . " Chars To Trusted Content.\n" );

                    $trusted .= $protect[0];
                    $curpos   = $protect[1];

                    continue;
                }
            }

            /**
             * Run All Other Checks
             * If We Have A Valid Tag, Append It To Trusted.
             */

            if ( (bool)FALSE === $tagname )
            {
                //$this -> spew( "$me: <h4>Removing invalid tag <i>{$tagname}</i></h4>\n" );
            }
            else
            {
                if ( (bool)FALSE === $skip_content && ! in_array( $tagname, $this -> self_closing_tags ) )
                {
                    $tagtrack = $this -> tagprint( $print_tag[ $tagname ], array(), 2, $delim );
                }

                $this -> processTag(   $skip_content
                                     , $tagname
                                     , $tagtype
                                     , $open_tags
                                     , $attary
                                     , $tagtrack
                                   );

                if ( (bool)FALSE !== $tagname && (bool)FALSE === $skip_content )
                {
                    //$this -> spew( "$me: Swapping Tag Delimiters And Appending Tag To Trusted.\n" );

                    $trusted .= $this -> tagprint( $print_tag[ $tagname ], $attary, $tagtype, $delim );
                }
            }

            $curpos = $gt + 1;

        } // End While/Do

        //$this -> spew( "$me: Appending any leftover content\n" );

        $trusted .= $this -> freeContent( $body, $this -> body_length, $curpos, TRUE );

        if ( (bool)TRUE === $this -> force_tag_closing && ! empty( $open_tags ) )
        {
            $this -> closeTags( $trusted, $open_tags );
        }

        // Encoding Specific Adjustments

        if ( $encode )
        {
            if ( $this -> addsig )
            {
                $trusted .= "\n[addsig /]\n";
            }

            if ( $edited )
            {
                $trusted .= '<br /><br /><div class="forumsig">[ ' . translate( 'This message was edited by:' ) . ' ' . $editby . ' ' . translate( 'on' ) . ' ';

                $trusted .= strftime( translate( 'forumdatestring2' ), _MPN_TIMESTAMP );

                $trusted .= " ]</div>";
            }

            // Adjust Formatting For Screen Display

            $this -> adjustFormat( $trusted, 3, 9 );
        }

        return $trusted;

    } // End Method BBConversion

} // End Child Class

?>